/**
 * @license
 * SPDX-License-Identifier: Apache-2.0
 */

// Ensure QRCode is globally available from the script tag in index.html
declare var QRCode: {
    new (element: HTMLElement | string, optionsOrText?: string | object): {
        makeCode: (text: string) => void;
        clear: () => void;
    };
    CorrectLevel: { L: number; M: number; Q: number; H: number; };
};

// Declare html2canvas
declare function html2canvas(element: HTMLElement, options?: any): Promise<HTMLCanvasElement>;


interface VCardData {
    firstName: string;
    lastName: string;
    position?: string;
    company?: string;
    email?: string;
    phone?: string;
    linkedin?: string;
}

function generateVCardString(data: VCardData): string {
    let vCard = "BEGIN:VCARD\n";
    vCard += "VERSION:3.0\n";
    vCard += `N:${data.lastName};${data.firstName}\n`;
    vCard += `FN:${data.firstName} ${data.lastName}\n`;
    if (data.company) vCard += `ORG:${data.company}\n`;
    if (data.position) vCard += `TITLE:${data.position}\n`;
    if (data.phone) vCard += `TEL;TYPE=WORK,VOICE:${data.phone}\n`;
    if (data.email) vCard += `EMAIL:${data.email}\n`;
    if (data.linkedin) vCard += `URL;TYPE=linkedin:${data.linkedin}\n`;
    vCard += "END:VCARD";
    return vCard;
}

document.addEventListener('DOMContentLoaded', () => {
    const vcardForm = document.getElementById('vcardForm') as HTMLFormElement;
    const generateButton = document.getElementById('generateButton') as HTMLButtonElement;
    const cardFullNameElement = document.getElementById('cardFullName') as HTMLDivElement;
    const qrCodeContainerElement = document.getElementById('qrCodeContainer') as HTMLDivElement;
    const statusMessageElement = document.getElementById('statusMessage') as HTMLParagraphElement;
    const downloadButton = document.getElementById('downloadButton') as HTMLButtonElement;
    const businessCardElement = document.getElementById('businessCard') as HTMLDivElement;


    if (!vcardForm || !generateButton || !cardFullNameElement || !qrCodeContainerElement || !statusMessageElement || !downloadButton || !businessCardElement) {
        console.error('One or more essential DOM elements are missing.');
        if (statusMessageElement) {
            statusMessageElement.textContent = 'Error: Essential page elements are missing. App cannot start.';
            statusMessageElement.className = 'error';
        }
        if(generateButton) generateButton.disabled = true;
        if(vcardForm) vcardForm.style.opacity = '0.5'; vcardForm.style.pointerEvents = 'none';
        return;
    }
    
    // Set initial state for card name
    cardFullNameElement.textContent = "Your Name";
    downloadButton.style.display = 'none'; // Hide download button initially

    // Set initial placeholder text in the qrCodeContainer
    const initialPlaceholderText = qrCodeContainerElement.dataset.initialPlaceholder || "QR code will appear here.";
    qrCodeContainerElement.innerHTML = `<p class="placeholder-text">${initialPlaceholderText}</p>`;

    // let qrCodeInstance: any | null = null; // qrCodeInstance is not actively used for .clear() or .makeCode() after instantiation in the current flow


    vcardForm.addEventListener('submit', async (event) => {
        event.preventDefault();
        statusMessageElement.textContent = '';
        statusMessageElement.className = '';
        generateButton.disabled = true;
        downloadButton.style.display = 'none';
        
        qrCodeContainerElement.innerHTML = `<p class="placeholder-text">Generating...</p>`;

        try {
            if (typeof QRCode === 'undefined' || typeof QRCode.CorrectLevel === 'undefined') {
                throw new Error('QR Code library (qrcode.js) is not loaded correctly.');
            }

            const formData = new FormData(vcardForm);
            const data: VCardData = {
                firstName: formData.get('firstName') as string,
                lastName: formData.get('lastName') as string,
                position: formData.get('position') as string || undefined,
                company: formData.get('company') as string || undefined,
                email: formData.get('email') as string || undefined,
                phone: formData.get('phone') as string || undefined,
                linkedin: formData.get('linkedin') as string || undefined,
            };

            if (!data.firstName || !data.lastName) {
                throw new Error('First Name and Last Name are required.');
            }

            const vCardString = generateVCardString(data);
            
            cardFullNameElement.textContent = `${data.firstName} ${data.lastName}`;

            qrCodeContainerElement.innerHTML = ''; 

            new QRCode(qrCodeContainerElement, {
                text: vCardString,
                width: 160, 
                height: 160, 
                colorDark: "#000000",
                colorLight: "#ffffff",
                correctLevel: QRCode.CorrectLevel.M
            });
            
            const imgElement = qrCodeContainerElement.querySelector('img');
            if (imgElement) {
                imgElement.alt = `vCard QR Code for ${data.firstName} ${data.lastName}`;
                // Removed explicit style.width and style.height settings
                // The library sets the dimensions based on options; CSS provides fallback/default.
                imgElement.style.borderRadius = '2px'; 
                imgElement.setAttribute('aria-label', `vCard QR Code for ${data.firstName} ${data.lastName}`);
            } else {
                 // Check for canvas as well, as qrcode.js might render to canvas by default on some paths
                 const canvasElement = qrCodeContainerElement.querySelector('canvas');
                 if (canvasElement) {
                    canvasElement.setAttribute('aria-label', `vCard QR Code for ${data.firstName} ${data.lastName}`);
                    // Apply border radius if needed, e.g., canvasElement.style.borderRadius = '2px';
                 } else {
                    console.warn("QR code image or canvas element not found after generation. Alt/aria-label text not set.");
                 }
            }

            statusMessageElement.textContent = 'Business card generated successfully!';
            statusMessageElement.className = 'success';
            downloadButton.style.display = 'inline-block'; // Show download button

        } catch (error: any) {
            console.error('Error processing vCard:', error);
            statusMessageElement.textContent = `Error: ${error.message || 'Could not generate card.'}`;
            statusMessageElement.className = 'error';
            cardFullNameElement.textContent = "Your Name"; 
            downloadButton.style.display = 'none';
            
            // const errorPlaceholderText = qrCodeContainerElement.dataset.initialPlaceholder || "Failed. Try again."; // This line seems redundant
            qrCodeContainerElement.innerHTML = `<p class="placeholder-text error-text">${error.message || 'Failed to generate. Please try again.'}</p>`;

        } finally {
            generateButton.disabled = false;
        }
    });

    downloadButton.addEventListener('click', async () => {
        if (typeof html2canvas === 'undefined') {
            console.error('html2canvas library is not loaded.');
            statusMessageElement.textContent = 'Error: Download feature not available (library missing).';
            statusMessageElement.className = 'error';
            return;
        }

        const originalButtonText = downloadButton.textContent;
        downloadButton.textContent = 'Downloading...';
        downloadButton.disabled = true;
        statusMessageElement.textContent = 'Preparing download...';
        statusMessageElement.className = '';


        try {
            const canvas = await html2canvas(businessCardElement, { 
                scale: 2, // Higher scale for better quality
                useCORS: true, 
                backgroundColor: '#fdfdfd' // Match card background
            });
            const imageDataUrl = canvas.toDataURL('image/png');
            const link = document.createElement('a');
            link.href = imageDataUrl;
            
            const firstNameInput = document.getElementById('firstName') as HTMLInputElement | null;
            const lastNameInput = document.getElementById('lastName') as HTMLInputElement | null;
            const fName = firstNameInput?.value || 'Card';
            const lName = lastNameInput?.value || 'User';
            link.download = `business-card-${fName.replace(/\s+/g, '_')}-${lName.replace(/\s+/g, '_')}.png`;
            
            document.body.appendChild(link);
            link.click();
            document.body.removeChild(link);
            statusMessageElement.textContent = 'Download started!';
            statusMessageElement.className = 'success';

        } catch (err: any) {
            console.error('Error generating image from card:', err);
            statusMessageElement.textContent = `Error: Could not download card image. ${err.message || ''}`;
            statusMessageElement.className = 'error';
        } finally {
            downloadButton.textContent = originalButtonText;
            downloadButton.disabled = false;
        }
    });
});

// Simple check for offline functionality - this app is client-side only once loaded.
if ('serviceWorker' in navigator) {
    window.addEventListener('load', () => {
        // Example: navigator.serviceWorker.register('/sw.js').then...
        // For this example, actual service worker (sw.js) is not provided.
        // console.log('Service Worker registration placeholder.');
    });
}
